package com.foreverwin.mesnac.anomaly.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.anomaly.model.AbnormalMessage;
import com.foreverwin.mesnac.anomaly.service.AbnormalMessageService;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *
 * @author 赵嘉伟
 * @since 2021-07-22
 */
@RestController
@RequestMapping("/Z-ABNORMAL-MESSAGE")
public class AbnormalMessageController {

    @Autowired
    public AbnormalMessageService abnormalMessageService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getAbnormalMessageById(@PathVariable String id) {
        return R.ok( abnormalMessageService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getAbnormalMessageList(AbnormalMessage abnormalMessage){
        List<AbnormalMessage> result;
        QueryWrapper<AbnormalMessage> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalMessage);
        result = abnormalMessageService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<AbnormalMessage> frontPage, AbnormalMessage abnormalMessage){
        IPage result;
        QueryWrapper<AbnormalMessage> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalMessage);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(AbnormalMessage::getHandle, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getSite, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getAbnormalBillBo, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getType, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getMessageType, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getSendUserGroup, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getSendUser, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getContent, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getUpUserGroup, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getUpUser, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getUpMessageType, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getAbnormalNode, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getStatus, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getCreatedUser, frontPage.getGlobalQuery())
                .or().like(AbnormalMessage::getModifiedUser, frontPage.getGlobalQuery())
    );
        }
        result = abnormalMessageService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param abnormalMessage  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody AbnormalMessage abnormalMessage) {
        return R.ok(abnormalMessageService.save(abnormalMessage));
    }

    /**
     * 修改
     * @param abnormalMessage  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody AbnormalMessage abnormalMessage) {
        return R.ok(abnormalMessageService.updateById(abnormalMessage));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(abnormalMessageService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(abnormalMessageService.removeByIds(ids));
    }
}