package com.foreverwin.mesnac.common.service.impl;


import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONString;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.common.dto.LabelPrintDto;
import com.foreverwin.mesnac.common.model.PrintLog;
import com.foreverwin.mesnac.common.mapper.PrintLogMapper;
import com.foreverwin.mesnac.common.service.PrintLogService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author Philip
 * @since 2021-07-16
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class PrintLogServiceImpl extends ServiceImpl<PrintLogMapper, PrintLog> implements PrintLogService {

    @Value("${print.server}")
    private String printServer;

    @Autowired
    private PrintLogMapper printLogMapper;

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void savePrintLog(String site, String user, PrintLog printLog) {
        LocalDateTime dateTime = LocalDateTime.now();

        printLog.setSite(site);
        printLog.setCreateUser(user);
        printLog.setCreatedDateTime(dateTime);
        printLog.setModifyUser(user);
        printLog.setModifiedDateTime(dateTime);
        this.save(printLog);
    }

    @Override
    public void labelPrint(String site, String user, List<PrintLog> printList) {
        if (printList == null || printList.size() <= 0) {
            return;
        }

        List<LabelPrintDto> labelPrintDtoList = new ArrayList<>();
        for (PrintLog printLog : printList) {
            LabelPrintDto labelPrintDto = new LabelPrintDto();
            labelPrintDto.setPrinter(printLog.getPrintName());
            labelPrintDto.setPrintTemplate(printLog.getPrintTemplate());
            labelPrintDto.setItem(printLog.getItem());
            labelPrintDto.setItemDescription(printLog.getItemDescription());
            labelPrintDtoList.add(labelPrintDto);
        }

        HttpUtil.post(printServer, JSON.toJSONString(labelPrintDtoList), 30000);
    }
}